;=-------------------------------------------------------------------------=
;
;			  Enhanced Master Boot Record 
;
;     Copyright (c) 1999-2000 by Thomas Kjoernes, all rights reserved.
;
;=-------------------------------------------------------------------------=

	INCLUDE	std.inc
	INCLUDE	mbr.inc

	.386p

; MBR code

bootSegment SEGMENT USE16 'boot'

	ASSUME	cs:bootSegment, ds:bootSegment

	cld

	xor	ax, ax
	mov	ss, ax
	mov	sp, 7C00h

	mov	ds, ax
	mov	es, ax

	; relocate ourself down to 0060:0000

	mov	cx, 0100h
	mov	si, 7C00h
	mov	di, 0600h
	rep	movsw

	JMPF	0060h, bootEntryPoint

bootEntryPoint:

	push	cs
	pop	ds

	mov	mbrBootDrive, dl

	test	BYTE PTR es:[417h], 08h
	jnz	findPartitionMain

	mov	ax, 020Fh
	mov	bx, 0800h
	mov	cx, 0002h
	mov	dx, 0080h
	int	13h

	cmp	extMagicTag, 0AA55BEEFh	
	jne	findPartitionMain

	JMPF	0060h, 0200h

findPartitionMain:
	mov	cl, 4
	mov	si, OFFSET mbrPTable

findPartitionNext:
;*	test	[si].mbrBootFlag, MBR_ENTRY_BOOTABLE
	cmp	[si].mbrBootFlag, ch
	jl	findPartitionDone

	add	si, SIZE MBR_ENTRY
	loop	findPartitionNext

	int	18h

findPartitionDone:
	mov	cx, [si][2]
	mov	dx, [si]

	cmp	dh, 0FFh
	jne	findPartitionNorm

	cmp	cx, 0FFFFh
	jne	findPartitionNorm

	push	LARGE NULL
	push	[si].mbrPartitionStart
	push	es
	push	bx
	push	SMALL 1
	push	SMALL 16

	push	ss
	pop	ds

	mov	si, sp

	mov	ah, 42h
	int	13h

	add	sp, 16
	jmp	findPartitionExec

findPartitionNorm:
	mov	ax, 0201h
	mov	bx, 7C00h
	int	13h

findPartitionExec:
	JMPF	0000h, 7C00h

mbrBootDrive	DB	?

	MBR_TABLE_HERE

mbrPTable MBR_ENTRY 4 DUP (<>)

	MBR_MAGIC_HERE

	push	0B800h
	pop	es

	mov	ax, 1720h
	mov	cx, 2000
	xor	di, di
	rep	stosw

	push	0
	pop	es

	mov	ah, 02h
	mov	bh, 00h
	xor	dx, dx
	int	10h

	jmp	findPartitionMain

extMagicTag EQU DWORD PTR ds:[EXT_MAGIC_POS]
	EXT_MAGIC_HERE

bootSegment ENDS

; DOS code

codeSegment SEGMENT USE16 'code'

	ASSUME	cs:codeSegment, ds:codeSegment

mainEntryPoint:

	push	cs
	pop	ds

	push	cs
	pop	es

	mov	ax, 0201h
	mov	bx, OFFSET ourSectorBuf
	mov	cx, 0001h
	mov	dx, 0080h
	int	13h

	push	bootSegment
	pop	es

	mov	si, MBR_TABLE_POS + OFFSET ourSectorBuf
	mov	di, MBR_TABLE_POS
	mov	cx, 32
	rep	movsw

	mov	ax, 0302h
	xor	bx, bx
	mov	cx, 1
	mov	dx, 0080h
	int	13h

	mov	ax, 4C00h
	int	21h

ourSectorBuf DB 512 DUP (?)

codeSegment ENDS

	END	mainEntryPoint
