;---------------------------------------------------------------------------
;
;		MS-DOS FAT -- File Allocation Table Structures
;
;---------------------------------------------------------------------------

BOOT_MAGIC		EQU	0AA55h		; "U"
BOOT_MAGIC_POS		EQU	01FEh		;

;---------------------------------------------------------------------------
; Signature Placement
;---------------------------------------------------------------------------

BOOT_MAGIC_HERE MACRO
	ORG	BOOT_MAGIC_POS
	DW	BOOT_MAGIC
	ENDM

;---------------------------------------------------------------------------
; Various Equates
;---------------------------------------------------------------------------

FS_INFO_MAGIC		EQU	61417272h	; "rrAa"
FS_INFO_MAGIC_POS	EQU	01E4h		;

EXT_FLAGS_FAT_MASK	EQU	000Fh		;
EXT_FLAGS_FAT_MIRROR	EQU	0080h		;

FAT12_MIN_CLUSTERs	EQU	00000001h	; 1
FAT12_MAX_CLUSTERs	EQU	00000FFFh - 11	; 4084
FAT12_BAD		EQU	00000FF7h	;
FAT12_EOF		EQU	00000FF8h	;

FAT16_MIN_CLUSTERs	EQU	00000FFFh - 10	; 4085
FAT16_MAX_CLUSTERs	EQU	0000FFFFh - 11	; 65524
FAT16_BAD		EQU	0000FFF7h	;
FAT16_EOF		EQU	0000FFF8h	;

FAT32_MIN_CLUSTERs	EQU	0000FFFFh - 10	; 65525
FAT32_MAX_CLUSTERs	EQU	0FFFFFFFh - 11	; 268435444
FAT32_BAD		EQU	0FFFFFF7h	;
FAT32_EOF		EQU	0FFFFFF8h	;

;---------------------------------------------------------------------------
; BPB for FAT12	volumes
;---------------------------------------------------------------------------

BPB_FAT12		STRUC
bpbBytesPerSector	DW	?		; 0x0B
bpbSectorsPerCluster	DB	?		; 0x0D
bpbReservedSectors	DW	?		; 0x0E
bpbNumberOfFATs		DB	?		; 0x10
bpbRootEntries		DW	?		; 0x11
bpbTotalSectors		DW	?		; 0x13
bpbMedia		DB	?		; 0x15
bpbSectorsPerFAT	DW	?		; 0x16
bpbSectorsPerTrack	DW	?		; 0x18
bpbHeadsPerCylinder	DW	?		; 0x1A
bpbHiddenSectors	DD	?		; 0x1C
bpbTotalSectorsBig	DD	?		; 0x20
BPB_FAT12		ENDS

;---------------------------------------------------------------------------
; BPB for FAT16 volumes
;---------------------------------------------------------------------------

BPB_FAT16		STRUC
bpbBytesPerSector	DW	?		; 0x0B
bpbSectorsPerCluster	DB	?		; 0x0D
bpbReservedSectors	DW	?		; 0x0E
bpbNumberOfFATs		DB	?		; 0x10
bpbRootEntries		DW	?		; 0x11
bpbTotalSectors		DW	?		; 0x13
bpbMedia		DB	?		; 0x15
bpbSectorsPerFAT	DW	?		; 0x16
bpbSectorsPerTrack	DW	?		; 0x18
bpbHeadsPerCylinder	DW	?		; 0x1A
bpbHiddenSectors	DD	?		; 0x1C
bpbTotalSectorsBig	DD	?		; 0x20
BPB_FAT16		ENDS

;---------------------------------------------------------------------------
; BPB for FAT32 volumes
;---------------------------------------------------------------------------

BPB_FAT32		STRUC
bpbBytesPerSector	DW	?		; 0x0B
bpbSectorsPerCluster	DB	?		; 0x0D
bpbReservedSectors	DW	?		; 0x0E
bpbNumberOfFATs		DB	?		; 0x10
bpbRootEntries		DW	?		; 0x11
bpbTotalSectors		DW	?		; 0x13
bpbMedia		DB	?		; 0x15
bpbSectorsPerFAT	DW	?		; 0x16
bpbSectorsPerTrack	DW	?		; 0x18
bpbHeadsPerCylinder	DW	?		; 0x1A
bpbHiddenSectors	DD	?		; 0x1C
bpbTotalSectorsBig	DD	?		; 0x20
bpb32SectorsPerFAT	DD	?		; 0x24
bpb32Flags		DW	?		; 0x28
bpb32Version		DW	?		; 0x2A
bpb32RootCluster	DD	?		; 0x2C
bpb32InfoSector		DW	?		; 0x30
bpb32BootBackupStart	DW	?		; 0x32
bpb32Reserved		DB	12 DUP (?)	; 0x34
BPB_FAT32		ENDS

;---------------------------------------------------------------------------
; Boot Sector layout for FAT12
;---------------------------------------------------------------------------

BS_FAT12		STRUC
bsJmp			DB	3 DUP (?)	; 0x00
bsOemName		DB	8 DUP (?)	; 0x03
bsFAT12			BPB_FAT12 <>		; 0x0B
bsDriveNumber		DB	?		; 0x24
bsUnused		DB	?		; 0x25
bsExtBootSignature	DB	?		; 0x26
bsSerialNumber		DD	?		; 0x27
bsVolumeLabel		DB	"NO NAME    "	; 0x2B
bsFileSystem		DB	"FAT12   "	; 0x36
bsBootCode		DB	450 DUP (?)	; 0x3E
BS_FAT12		ENDS

;---------------------------------------------------------------------------
; Boot Sector layout for FAT16
;---------------------------------------------------------------------------

BS_FAT16		STRUC
bsJmp			DB	3 DUP (?)	; 0x00
bsOemName		DB	8 DUP (?)	; 0x03
bsFAT16			BPB_FAT16 <>		; 0x0B
bsDriveNumber		DB	?		; 0x24
bsUnused		DB	?		; 0x25
bsExtBootSignature	DB	?		; 0x26
bsSerialNumber		DD	?		; 0x27
bsVolumeLabel		DB	"NO NAME    "	; 0x2B
bsFileSystem		DB	"FAT16   "	; 0x36
bsBootCode		DB	450 DUP (?)	; 0x3E
BS_FAT16		ENDS

;---------------------------------------------------------------------------
; Boot Sector layout for FAT32
;---------------------------------------------------------------------------

BS_FAT32		STRUC
bsJmp			DB	3 DUP (?)	; 0x00
bsOemName		DB	8 DUP (?)	; 0x03
bsFAT32			BPB_FAT32 <>		; 0x0B
bs32DriveNumber		DB	?		; 0x40
bs32Unused		DB	?		; 0x41
bs32ExtBootSignature	DB	?		; 0x42
bs32SerialNumber	DD	?		; 0x43
bs32VolumeLabel		DB	"NO NAME    "	; 0x47
bs32FileSystem		DB	"FAT32   "	; 0x52
bs32BootCode		DB	422 DUP (?)	; 0x5A
BS_FAT32		ENDS

;---------------------------------------------------------------------------
; FAT32 File System Information Structure
;---------------------------------------------------------------------------

FS_INFO			STRUC
fsInfoSignature		DD	?		; 0x00
fsInfoFreeClusterCount	DD	?		; 0x04
fsInfoNextCluster	DD	?		; 0x08
fsInfoReserved		DD	3 DUP (?)	; 0x0C
FS_INFO			ENDS

;---------------------------------------------------------------------------
; Directory Entry format
;---------------------------------------------------------------------------

DIR_ENTRY		STRUC
dirName			DB	11 DUP (?)	; 0x00
dirAttr			DB	?		; 0x0B
dirReserved		DB	?		; 0x0C
dirCreate10MS		DB	?		; 0x0D
dirCreateTime		DW	?		; 0x0E
dirCreateDate		DW	?		; 0x10
dirAccessDate		DW	?		; 0x12
dirClusterHigh		DW	?		; 0x14
dirTime			DW	?		; 0x16
dirDate			DW	?		; 0x18
dirCluster		DW	?		; 0x1A
dirSize			DD	?		; 0x1C
DIR_ENTRY		ENDS

;---------------------------------------------------------------------------
; LFN (Long File Name) Entry format
;---------------------------------------------------------------------------

LFN_ENTRY		STRUC
lfnSequence		DB	?		; 0x00
lfnName1		DW	5 DUP (?)	; 0x01
lfnAttr			DB	?		; 0x0B
lfnReserved		DB	?		; 0x0C
lfnChecksum		DB	?		; 0x0D
lfnName2		DW	6 DUP (?)	; 0x0E
lfnCluster		DW	?		; 0x1A
lfnName3		DW	2 DUP (?)	; 0x1C
LFN_ENTRY		ENDS

;---------------------------------------------------------------------------
; LFN Specific Equates
;---------------------------------------------------------------------------

LFN_MAX_NAME		EQU	251		; trial-and-failure (Win95)
LFN_MAX_PATH		EQU	260		;

LFN_TAG_ATTR		EQU	0Fh
LFN_TAG_MASK		EQU	0Fh

LFN_SEQ_DELETED		EQU	80h
LFN_SEQ_LAST		EQU	40h
LFN_SEQ_MASK		EQU	3Fh

