; Operand Structure

IA32_OPERAND	STRUC
iaType		DB	?	; reg/mem/imm/rel/ofs/ptr
iaSize		DB	?	; byte/word/dword/fword/qword/tbyte
iaMode		DB	?	; scale, index, base, disp
iaRegs		DB	?	; register(s), depends on *OperandType*
iaData		DD	?	; imm, disp, ptr
IA32_OPERAND	ENDS

; Opcode Decoding Structure

IA32_OPCODE	STRUC
iaFlags		DB	?	; various flags
iaBytes		DB	?	; updated by decoder
iaIndex		DB	?	; opcode string index
iaTable		DB	?	; opcode string table
iaPrefixes	DB	?	; prefix bit flags
iaOverride	DB	?	; segment override
iaOperands	DB	?	; number of operands
iaReserved	DB	?	; unused padding
iaOperandData	DQ	?, ?, ?	;
IA32_OPCODE	ENDS

; None

NONE		EQU	0

; Flags

FLAGS_USE32	EQU	11000000b
FLAGS_USE16	EQU	00000000b
FLAGS_PTR	EQU	00100000b
FLAGS_API	EQU	00010000b

; Prefixes

PREFIX_SIZE	EQU	10000000b
PREFIX_ADDR	EQU	01000000b
PREFIX_LOCK	EQU	00100000b
PREFIX_REP	EQU	00010000b
PREFIX_REPE	EQU	00010000b
PREFIX_REPZ	EQU	00010000b
PREFIX_REPNE	EQU	00001000b
PREFIX_REPNZ	EQU	00001000b

; Operand Types

TYPE_REG	EQU	0		; register
TYPE_MEM	EQU	1		; memory
TYPE_IMM	EQU	2		; immediate
TYPE_REL	EQU	3		; relative
TYPE_OFS	EQU	4		; offset
TYPE_PTR	EQU	5		; pointer (seg:ofs)
TYPE_STR_S	EQU	6		; string (source)
TYPE_STR_D	EQU	7		; string (destination)
TYPE_API	EQU	8		; APIcall argument

; Operand Size

SIZE_BYTE	EQU	1
SIZE_WORD	EQU	2
SIZE_DWORD	EQU	4
SIZE_FWORD	EQU	6
SIZE_QWORD	EQU	8
SIZE_TBYTE	EQU	10

; Addressing Modes

MODE_BASE	EQU	10000000b
MODE_INDEX	EQU	01000000b
MODE_SCALE	EQU	00100000b
MODE_DISP	EQU	00010000b

; Scalings

SCALE_1		EQU	0
SCALE_2		EQU	1
SCALE_4		EQU	2
SCALE_8		EQU	3
SCALE_MASK	EQU	00001100b

; Displacements

DISP_8		EQU	0
DISP_16		EQU	1
DISP_32		EQU	2
DISP_64		EQU	3
DISP_MASK	EQU	00000011b

; Register Masks

REGS_BASE	EQU	11110000b
REGS_INDEX	EQU	00001111b

; Readability

BASE_BIN	EQU	2
BASE_OCT	EQU	8
BASE_DEC	EQU	10
BASE_HEX	EQU	16

; String Table Selectors

STRING_MAP1	EQU	00h
STRING_MAP2	EQU	01h
STRING_DUAL	EQU	02h
STRING_FAKE	EQU	03h
STRING_GRP1	EQU	04h
STRING_GRP2	EQU	05h
STRING_GRP3	EQU	06h
STRING_GRP4	EQU	07h
STRING_GRP5	EQU	08h
STRING_GRP6	EQU	09h
STRING_GRP7	EQU	0Ah
STRING_GRP8	EQU	0Bh
STRING_GRP9	EQU	0Ch
STRING_GRPA_W	EQU	0Dh
STRING_GRPA_D	EQU	0Eh
STRING_GRPA_Q	EQU	0Fh
STRING_GRPB	EQU	10h
STRING_GRPC	EQU	11h

; 16-bit General Purpose Registers

REG_AX		EQU	00h
REG_CX		EQU	01h
REG_DX		EQU	02h
REG_BX		EQU	03h
REG_SP		EQU	04h
REG_BP		EQU	05h
REG_SI		EQU	06h
REG_DI		EQU	07h

; 32-bit General Purpose Registers

REG_EAX		EQU	08h
REG_ECX		EQU	09h
REG_EDX		EQU	0Ah
REG_EBX		EQU	0Bh
REG_ESP		EQU	0Ch
REG_EBP		EQU	0Dh
REG_ESI		EQU	0Eh
REG_EDI		EQU	0Fh

; 8-bit General Purpose Registers

REG_AL		EQU	10h
REG_CL		EQU	11h
REG_DL		EQU	12h
REG_BL		EQU	13h
REG_AH		EQU	14h
REG_CH		EQU	15h
REG_DH		EQU	16h
REG_BH		EQU	17h

; Segment Registers

REG_ES		EQU	18h
REG_CS		EQU	19h
REG_SS		EQU	1Ah
REG_DS		EQU	1Bh
REG_FS		EQU	1Ch
REG_GS		EQU	1Dh

; Control Registers

REG_CR0		EQU	20h
REG_CR1		EQU	21h
REG_CR2		EQU	22h
REG_CR3		EQU	23h
REG_CR4		EQU	24h
REG_CR5		EQU	25h
REG_CR6		EQU	26h
REG_CR7		EQU	27h

; Debug Registers

REG_DR0		EQU	28h
REG_DR1		EQU	29h
REG_DR2		EQU	2Ah
REG_DR3		EQU	2Bh
REG_DR4		EQU	2Ch
REG_DR5		EQU	2Dh
REG_DR6		EQU	2Eh
REG_DR7		EQU	2Fh

; Test Registers

REG_TR0		EQU	30h
REG_TR1		EQU	31h
REG_TR2		EQU	32h
REG_TR3		EQU	33h
REG_TR4		EQU	34h
REG_TR5		EQU	35h
REG_TR6		EQU	36h
REG_TR7		EQU	37h

; MMX Registers

REG_MM0		EQU	38h
REG_MM1		EQU	39h
REG_MM2		EQU	3Ah
REG_MM3		EQU	3Bh
REG_MM4		EQU	3Ch
REG_MM5		EQU	3Dh
REG_MM6		EQU	3Eh
REG_MM7		EQU	3Fh

; FPU Registers

REG_ST0		EQU	40h
REG_ST1		EQU	41h
REG_ST2		EQU	42h
REG_ST3		EQU	43h
REG_ST4		EQU	44h
REG_ST5		EQU	45h
REG_ST6		EQU	46h
REG_ST7		EQU	47h

