;---------------------------------------------------------------------------
; I/O Request Block -- Generic
;---------------------------------------------------------------------------
;
; The following structure defines the layout of a generic IRB. The devices
; are called with a pointer to this structure when asked to perform async
; operations.
;
; irbLink
;
;	This field will either point to the next IRB in the queue or
;	terminate the chain of IRBs with a NULL.
;
; irbCommand
;
;	This field simply specifies a device command code in the range
;	00h-FFh, of which some are generic and apply to all devices, while 
;	others are device class specific.
;
; irbStatus
;
;	Request Status, similar to irbCommand with regards to 
;	generic/specific.
;
; irbType
;
;	Request Type, similar to irbCommand, may or may not apply to all 
;	commands or all device classes.
;
; irbFlags
;
;	Request Flags, similar to irbCommand or irbType. Contains special
;	On/Off-style request options.
;
; irbDRD
;
;	This field contains "Device Reference Data". This is usually some
;	sort of identification value or a pointer to some device specific
;	control structure.
;
; irbRID
;
;	This field contains a unique "Requestor ID". Basically this field 
;	will contain the Thread ID for the currently executing thread.
;	
; irbArg
;
;	This field contains request specific arguments. Each class of
;	devices will have argument equates defined for each supported
;	command that needs arguments.
;
;---------------------------------------------------------------------------

IRB			STRUC
irbLink			DD	?
irbCommand		DB	?
irbStatus		DB	?
irbType			DB	?
irbFlags		DB	?
irbDRD			DD	?
irbRID			DD	?
irbArg			DD	4 DUP (?)
IRB			ENDS

;---------------------------------------------------------------------------
; IRB Arguments
;---------------------------------------------------------------------------

ARG_1			EQU	0000h
ARG_2			EQU	0004h
ARG_3			EQU	0008h
ARG_4			EQU	000Ch

;---------------------------------------------------------------------------
; IRB Arguments for DISK class drivers
;---------------------------------------------------------------------------

; Query Capacity

irbCapacityLo		EQU	irbArg[ARG_1]
irbCapacityHi		EQU	irbArg[ARG_2]

; Query Geometry

irbCylinders		EQU	irbArg[ARG_1]
irbHeads		EQU	irbArg[ARG_2]
irbSectors		EQU	irbArg[ARG_3]
irbSectorSize		EQU	irbArg[ARG_4]

; Block Read/Write

irbBlockBuf		EQU	irbArg[ARG_1]
irbBlockCount		EQU	irbArg[ARG_2]
irbBlockStartLo		EQU	irbArg[ARG_3]
irbBlockStartHi		EQU	irbArg[ARG_4]
