;---------------------------------------------------------------------------
;
; 		    Driver Library IFS Support Routines
;
;     Copyright (c) 1999-2001 by Thomas Kjoernes, all rights reserved.
;
;
;	API Routines:
;
;	- KERNEL_IfsFileOpen
;	- KERNEL_IfsFileClose
;	- KERNEL_IfsFileRead	
;	- KERNEL_IfsFileWrite
;	- KERNEL_IfsFileGetPos
;	- KERNEL_IfsFileSetPos
;
;---------------------------------------------------------------------------

	INCLUDE	std.inc
	INCLUDE	stdgroup.inc
	INCLUDE	mzexe.inc
	INCLUDE	mzlib.inc

	.386p

	; Exports

	PUBLIC	ifsFileOpen
	PUBLIC	ifsFileClose
	PUBLIC	ifsFileRead
	PUBLIC	ifsFileWrite
	PUBLIC	ifsFileGetPos
	PUBLIC	ifsFileSetPos

	; Startup/Shutdown

	PUBLIC	ifsInit
	PUBLIC	ifsDone

codeSegment SEGMENT

;---------------------------------------------------------------------------
;	ifsFileOpen
;
;	Entry:
;		esi = pointer to ASCIIZ filename
;	Exit:
;		eax = file handle
;
DefProc ifsFileOpen

	push	ebx
	push	ecx
	push	esi
	push	edi

	lodsb				;
	cmp	al, "*"			; embedded object?
	jne	ifsFileOpenFail		;

	mov	ecx, 8
	mov	edi, OFFSET ifsNameBuf

	push	edi

ifsFileOpenCopy:
	lodsb				;
	test	al, al			; null?
	jnz	ifsFileOpenSkip		;

	dec	esi			; esi -> null

ifsFileOpenSkip:
	cmp	al, "a"			; below lowercase range?
	jb	ifsFileOpenSave		;
	cmp	al, "z"			; above lowercase range?
	ja	ifsFileOpenSave		;
	sub	al, " "			; upper...

ifsFileOpenSave:
	stosb				;
	dec	ecx			; store character
	jnz   	ifsFileOpenCopy		;

	pop	edi

	mov	ebx, ifsLibraryBase	; eax -> Library
	test	ebx, ebx		; null?
	jz	ifsFileOpenFail		;

	mov	esi, ebx		; esi -> Library

	cmp	[esi].mzSignature, MZ_EXE_MAGIC
	jne	ifsFileOpenFail

	add	esi, [esi].mzExeLibHeader

	cmp	[esi].mzLibSignature, MZ_LIB_MAGIC
	jne	ifsFileOpenFail

	mov	ecx, [esi].mzLibCount	; any entries?
	jecxz	ifsFileOpenFail		;

	add	esi, SIZE MZ_LIB	; esi -> first entry

ifsFileOpenFind:
	pushad				;
	mov	ecx, 2			;
	rep	cmpsd			; match?
	popad				;
	je	ifsFileOpenDone		;

	add	esi, SIZE MZ_OBJ	; esi -> next entry
	dec	ecx			; more?
	jnz	ifsFileOpenFind		;

	jmp	ifsFileOpenFail

ifsFileOpenDone:
	mov	eax, OFFSET ifsHandlesDef

ifsFileOpenScan:
	cmp	[eax].ifsHndID, NULL	; free handle?
	je	ifsFileOpenInit		;

	add	eax, SIZE IFS_FILE_HND
	cmp	eax, OFFSET ifsHandlesEnd
	jb	ifsFileOpenScan

	jmp	ifsFileOpenFail		; out of handles

ifsFileOpenInit:
	mov	[eax].ifsHndID, eax	; set ID
	mov	[eax].ifsHndPos, NULL	; rewind

	add	ebx, [esi].mzObjBase	; get object base
	mov	ecx, [esi].mzObjSize	; get object size

	mov	[eax].ifsHndBase, ebx	; set object base
	mov	[eax].ifsHndSize, ecx	; set object size

ifsFileOpenExit:
	pop	edi
	pop	esi
	pop	ecx
	pop	ebx
	ret

ifsFileOpenFail:
	stc				; failure!
	jmp	ifsFileOpenExit		;

EndProc ifsFileOpen

;---------------------------------------------------------------------------
;	ifsFileClose
;
;	Entry:
;		ebx = handle
;	Exit:
;		None
;
DefProc ifsFileClose

	cmp	[ebx].ifsHndID, ebx
	jne	ifsFileCloseFail

	xor	eax, eax
	mov	[ebx].ifsHndID, eax
	ret

ifsFileCloseFail:
	stc
	ret

EndProc ifsFileClose

;---------------------------------------------------------------------------
;	ifsFileRead
;
;	Entry:
;		ebx = handle
;		ecx = number of bytes to read
;		edx = buffer pointer
;	Exit:
;		eax = number of bytes read
;
DefProc ifsFileRead

	cmp	[ebx].ifsHndID, ebx
	jne	ifsFileReadFail

	cld

	push	ecx
	push	esi
	push	edi

	mov	edi, edx

	mov	eax, [ebx].ifsHndSize
	sub	eax, [ebx].ifsHndPos
	ja	ifsFileReadSkip

	xor	eax, eax
	jmp	ifsFileReadDone

ifsFileReadSkip:
	cmp	ecx, eax
	jna   	ifsFileReadMain
	mov	ecx, eax

ifsFileReadMain:
	mov	eax, ecx
	mov	esi, [ebx].ifsHndBase
	add	esi, [ebx].ifsHndPos
	rep	movsb

	add	[ebx].ifsHndPos, eax

ifsFileReadDone:
	clc

	pop	edi
	pop	esi
	pop	ecx
	ret

ifsFileReadFail:
	stc
	ret

EndProc ifsFileRead

;---------------------------------------------------------------------------
;	ifsFileWrite
;
;	Entry:
;		ebx = handle
;		ecx = number of bytes to write
;		edx = buffer pointer
;	Exit:
;		eax = number of bytes written
;
DefProc ifsFileWrite

	xor	eax, eax
	ret

EndProc ifsFileWrite

;---------------------------------------------------------------------------
;	ifsFileGetPos
;
;	Entry:
;		ebx = handle
;	Exit:
;		eax = file position
;
DefProc ifsFileGetPos

	mov	eax, [ebx].ifsHndPos
	ret

EndProc ifsFileGetPos

;---------------------------------------------------------------------------
;	ifsFileSetPos
;
;	Entry:
;		eax = file position
;		ebx = handle
;	Exit:
;		None
;
DefProc ifsFileSetPos

	mov	[ebx].ifsHndPos, eax
	ret

EndProc ifsFileSetPos

;---------------------------------------------------------------------------
;	ifsInit
;
;
;	Entry (Registers):
;
;		ebp = pointer to arguments on the stack
;
;	Entry (Stack Arguments):
;
;		DWORD	Base of Library
;		DWORD	Size of Library
;		DWORD	Base of Reclaim Area
;		DWORD	Size of Reclaim Area
;		DWORD	Base of MDT
;		DWORD	Size of MDT (number of entries)
;		DWORD	Base of PSP
;		DWORD	Base of Environment
;	Exit:
;		None
;
DefProc ifsInit

	mov	eax, [ebp]		; get library base
	mov	ifsLibraryBase, eax	;

	mov	eax, [ebp+4]		; get library size
	mov	ifsLibrarySize, eax	;

	ret

EndProc ifsInit

;---------------------------------------------------------------------------
;	ifsDone
;
;	Entry:
;		Done
;	Exit:
;	     	None
;
DefProc ifsDone
	ret
EndProc ifsDone

codeSegment ENDS

;---------------------------------------------------------------------------
; Data
;---------------------------------------------------------------------------

dataSegment SEGMENT

dataSegment ENDS

;---------------------------------------------------------------------------
; BSS Data
;---------------------------------------------------------------------------

bssSegment SEGMENT

IFS_FILE_HND	STRUC
ifsHndID	DD	?
ifsHndPos	DD	?
ifsHndBase	DD	?
ifsHndSize	DD	?
IFS_FILE_HND	ENDS

ifsLibraryBase	DD	?
ifsLibrarySize	DD	?

ifsNameBuf	DB	8 DUP (?)

ifsHandlesDef	LABEL	DWORD
	IFS_FILE_HND	4 DUP (<>)
ifsHandlesEnd	LABEL	DWORD

bssSegment ENDS

	END
