;---------------------------------------------------------------------------
;
;		     32-bit PM Virtual-8086 Mode Support
;
;     Copyright (c) 1999-2001 by Thomas Kjoernes, all rights reserved.
;
;
;	API Routines:
;
;	- KERNEL_V86GetExitProc
;	- KERNEL_V86SetExitProc
;
;---------------------------------------------------------------------------

	INCLUDE	std.inc
	INCLUDE	stdgroup.inc
	INCLUDE	kernel.api

	.386p

	; Publics

	PUBLIC	v86Int

	PUBLIC	v86GetExitProc
	PUBLIC	v86SetExitProc

	PUBLIC	v86Init
	PUBLIC	v86Done

codeSegment SEGMENT

	; Externs

;---------------------------------------------------------------------------
;	v86GetExitProc - 
;
;	Entry:
;		None
;	Exit:
;		ebx = V86 Exit Address
;
DefProc v86GetExitProc

	mov	ebx, v86ExitProc
	ret

EndProc v86GetExitProc

;---------------------------------------------------------------------------
;	v86SetExitProc - 
;
;	Entry:
;		ebx = New V86 Exit Address
;	Exit:
;		ebx = Old V86 Exit Address
;
DefProc v86SetExitProc

	xchg	ebx, v86ExitProc
	ret

EndProc v86SetExitProc

;---------------------------------------------------------------------------
;	v86GetESP - 
;
;	Entry:
;		ebp = pointer to V86_FRAME stack frame
;	Exit:
;		eax = flat V86 ESP
;
DefProc v86GetESP

	mov	eax, [ebp].v86_SS
	shl	eax, 4
	add	eax, [ebp].v86_ESP
	ret

EndProc v86GetESP

;---------------------------------------------------------------------------
;	v86GetEIP - 
;
;	Entry:
;		ebp = pointer to V86_FRAME stack frame
;	Exit:
;		eax = flat V86 EIP
;
DefProc v86GetEIP

	mov	eax, [ebp].v86_CS
	shl	eax, 4
	add	eax, [ebp].v86_EIP
	ret

EndProc v86GetEIP

;---------------------------------------------------------------------------
;	v86PushW - 
;
;	Entry (Registers):
;
;		ebp = pointer to V86_FRAME stack frame
;
;	Entry (Stack):
;		
;		DWORD value on stack (uses lower WORD only)
;	Exit:
;		None
;
DefProc v86PushW

	push	eax
	push	edx

	mov	edx, [esp+12]
	sub	[ebp].v86_ESP, 2

	call	v86GetESP
	mov	[eax], dx

	pop	edx
	pop	eax
	ret	4

EndProc v86PushW

;---------------------------------------------------------------------------
;	v86PushD - 
;
;	Entry (Registers):
;
;		ebp = pointer to V86_FRAME stack frame
;
;	Entry (Stack):
;		
;		DWORD value on stack
;	Exit:
;		None
;
DefProc v86PushD

	push	eax
	push	edx

	mov	edx, [esp+12]
	sub	[ebp].v86_ESP, 4

	call	v86GetESP
	mov	[eax], edx

	pop	edx
	pop	eax
	ret	4

EndProc v86PushD

;---------------------------------------------------------------------------
;	v86PopW - 
;
;	Entry:
;		ebp = pointer to V86_FRAME stack frame
;	Exit:
;		eax = value from V86 stack (zero extended)
;
DefProc v86PopW

	call	v86GetESP
	add	[ebp].v86_ESP, 2
	movzx	eax, WORD PTR [eax]
	ret

EndProc v86PopW

;---------------------------------------------------------------------------
;	v86PopD - 
;
;	Entry:
;		ebp = pointer to V86_FRAME stack frame
;	Exit:
;		eax = value from V86 stack
;
DefProc v86PopD

	call	v86GetESP
	add	[ebp].v86_ESP, 4
	mov	eax, [eax]
	ret

EndProc v86PopD

;---------------------------------------------------------------------------
;	v86Int - 
;
;	Entry:
;		None
;	Exit:
;		None
;
DefProc v86Int

	ret

EndProc v86Int

;---------------------------------------------------------------------------
;	v86GPFHandler - 
;
;	Entry (Registers):
;
;		eax = 0Dh
;		ebp = pointer to V86_FRAME stack frame
;
;	Entry (Stack Frame):
;
;		DWORD	EIP		(All)
;		DWORD	CS		(All)
;		DWORD	EFLAGS		(All)
;		DWORD	ESP		(V86/Ring3)
;		DWORD	SS		(V86/Ring3)
;		DWORD	ES		(V86)
;		DWORD	DS		(V86)
;		DWORD	FS		(V86)
;		DWORD	GS		(V86)
;	Exit:
;		Set CF==1 to indicate GPF exception was not handled
;
DefProc v86GPFHandler

	push	eax
	push	edx

	call	v86GetEIP

	mov	edx, [eax]

	cmp	dl, 0CDh
	je	v86GPFHandlerCDxx

	mov	dh, 03h

	cmp	dl, 0CCh
	je	v86GPFHandlerCDxx

	stc

v86GPFHandlerExit:
	pop	edx
	pop	eax
	ret

v86GPFHandlerCDxx:

	cmp	dh, 41h			; INT41?
	je	v86GPFHandlerCD41	;

	add	[ebp].v86_EIP, 2	; skip instruction

	push	[ebp].v86_EFLAGS
	call	v86PushW

	push	[ebp].v86_CS
	call	v86PushW

	push	[ebp].v86_EIP
	call	v86PushW

	movzx	eax, dh

	push	DWORD PTR [eax*4]
	pop	WORD PTR [ebp].v86_EIP
	pop	WORD PTR [ebp].v86_CS

	clc
	jmp	v86GPFHandlerExit

v86GPFHandlerCD41:
	jmp	v86ExitProc

EndProc v86GPFHandler

;---------------------------------------------------------------------------
;	v86IRQHandler - 
;
;	Entry (Registers):
;
;		eax = IRQ#
;		ebp = pointer to V86_FRAME stack frame
;
;	Entry (Stack Frame):
;
;		DWORD	EIP		(All)
;		DWORD	CS		(All)
;		DWORD	EFLAGS		(All)
;		DWORD	SS		(V86/Ring3)
;		DWORD	ESP		(V86/Ring3)
;		DWORD	ES		(V86)
;		DWORD	DS		(V86)
;		DWORD	FS		(V86)
;		DWORD	GS		(V86)
;	Exit:
;		Set CF==1 to indicate IRQ was not handled
;
DefProc v86IRQHandler

	push	eax

	push	[ebp].v86_EFLAGS
	call	v86PushW

	push	[ebp].v86_CS
	call	v86PushW

	push	[ebp].v86_EIP
	call	v86PushW

	APIcall	KERNEL_IrqGetVMIntByIRQ

	movzx	eax, al

	push	DWORD PTR [eax*4]
	pop	WORD PTR [ebp].v86_EIP
	pop	WORD PTR [ebp].v86_CS

	clc

	pop	eax
	ret

EndProc v86IRQHandler

;---------------------------------------------------------------------------
;	v86Init - 
;
;	Entry:
;		None
;	Exit:
;		None
;
DefProc v86Init

	mov	al, 0Dh
	mov	ebx, OFFSET v86GPFHandler
	APIcall	KERNEL_ExpSetHandlerVM

	ret

EndProc v86Init

;---------------------------------------------------------------------------
;	v86Done - 
;
;	Entry:
;		None
;	Exit:
;		None
;
DefProc v86Done

	ret

EndProc v86Done

codeSegment ENDS

;---------------------------------------------------------------------------
; Data
;---------------------------------------------------------------------------

dataSegment SEGMENT

dataSegment ENDS

;---------------------------------------------------------------------------
; BSS Data
;---------------------------------------------------------------------------

bssSegment SEGMENT

v86ExitProc	DD	?

bssSegment ENDS

	END

