;---------------------------------------------------------------------------
; For use with FAT12/16 boot sector sources only!
;---------------------------------------------------------------------------

testSegment SEGMENT USE16 'test'

	ASSUME	cs:testSegment

;---------------------------------------------------------------------------
;	dosDiskRead - 
;
;	Entry:
;		al = DOS drive number (0=A, 1=B, etc...)
;		cx = sector count
;	       ebx = sector #
;	     ds:dx = buffer address
;
dosDiskRead PROC NEAR

	pusha
	push	ds

	push	ds		; buffer SEG
	push	dx		; buffer OFS
	push	cx		; sector count
	push	ebx		; sector LBA #
	mov	bx, sp		; sp -> packet

	push	ss		; our packet is on the stack,
	pop	ds		; but DOS wants ds:dx as pointer

	mov	cx, -1		; specify extended read
	int	25h		; absolute disk read!
	pop	ax		; pop flags, long story...

	add	sp, 10		; size of disk I/O packet
	pop	ds
	popa
	ret

dosDiskRead ENDP

TEST_DRIVE		EQU	"F"
TEST_DRIVE_INT13	EQU	81h

testEntryPoint:

	int	3

	mov	ax, cs
	mov	ss, ax
	mov	sp, OFFSET ourStackTop

	mov	ah, 4Ah
	mov	bx, lastSegment
	sub	bx, codeSegment
	add	bx, 16
	int	21h

	push	cs
	pop	ds

	push	cs
	pop	es

	mov	si, 5

	mov	al, TEST_DRIVE - "A"
	xor	ebx, ebx
	mov	dx, OFFSET ourReadBuf
	mov	cx, 0001h
	call	dosDiskRead

	mov	ax, 3518h
	int	21h
	mov	oldInt18Ofs, bx
	mov	oldInt18Seg, es

	mov	ax, 3519h
	int	21h
	mov	oldInt19Ofs, bx
	mov	oldInt19Seg, es

	mov	ax, 2518h
	mov	dx, OFFSET ourInt18Handler
	int	21h

	mov	ax, 2519h
	mov	dx, OFFSET ourInt19Handler
	int	21h

	mov	ah, 48h		;
	mov	bx, -1		; query largest block
	int	21h		;

	mov	ah, 48h		; allocate DOS memory
	int	21h		;

	add	bx, ax		;
	sub	bx, 512 / 16	; es = bx - 512
	mov	es, bx		;

	; setup stack and base pointers

	sub	bx, 2048 / 16	; ss = bx - 2KB
	mov	ss, bx		;

	mov	sp, 2048	; 2KB stack
	mov	bp, sp		;

	; copy ourself to destination

	push	codeSegment	;
	pop	ds		;

	mov	cx, 256		; 256 words
	xor	si, si		;
	xor	di, di		;
	rep	movsw		;

	push	cs		;
	pop	ds		;

	mov	di, 3 + 8
	mov	si, OFFSET ourReadBuf + 3 + 8
	mov	cx, SIZE BPB_FAT12 - 8
	rep	movsb

	; jump to relocated code

	mov	dl, TEST_DRIVE_INT13

	push	es
	push	OFFSET mainEntryPoint
	retf

ourInt18Handler:
ourInt19Handler:

	mov	ax, 2518h
	lds	dx, cs:oldInt18
	int	21h

	mov	ax, 2519h
	lds	dx, cs:oldInt19
	int	21h

	mov	ax, 4C00h
	int	21h

oldInt18	LABEL	DWORD
oldInt18Ofs	DW	?
oldInt18Seg	DW	?

oldInt19	LABEL	DWORD
oldInt19Ofs	DW	?
oldInt19Seg	DW	?

ourReadBuf	DB	512 DUP (?)

ourStackDef	LABEL	WORD
		DB	512 DUP (?)
ourStackTop	LABEL	WORD

testSegment ENDS

lastSegment SEGMENT USE16

lastSegment ENDS

	END	testEntryPoint

;---------------------------------------------------------------------------
; For use with FAT32 boot sector source only!
;---------------------------------------------------------------------------

testSegment SEGMENT USE16 'test'

	ASSUME	cs:testSegment, ds:testSegment

TEST_DRIVE_INT13	EQU	80h

testEntryPoint:

	mov	ax, cs
	mov	ds, ax
	mov	ss, ax
	mov	sp, OFFSET ourStackTop

	mov	ah, 4Ah
	mov	bx, lastSegment
	sub	bx, codeSegment
	add	bx, 16
	int	21h

	mov	ax, 3518h
	int	21h
	mov	oldInt18Ofs, bx
	mov	oldInt18Seg, es

	mov	ax, 3519h
	int	21h
	mov	oldInt19Ofs, bx
	mov	oldInt19Seg, es

	mov	ax, 2518h
	mov	dx, OFFSET ourInt18Handler
	int	21h

	mov	ax, 2519h
	mov	dx, OFFSET ourInt19Handler
	int	21h

	push	cs
	pop	es

	; WARNING! This works for my FAT32 partition because it is
	; actually located @ CHS 0/1/1

	mov	ax, 0201h
	mov	bx, OFFSET ourReadBuf
	mov	cx, 0001h
	mov	dx, 0100h + TEST_DRIVE_INT13
	int	13h

	mov	ah, 48h		;
	mov	bx, -1		; query largest block
	int	21h		;

	mov	ah, 48h		; allocate DOS memory
	int	21h		;

	add	bx, ax		;
	sub	bx, 512 / 16	; es = bx - 512
	mov	es, bx		;

	; setup stack and base pointers

	sub	bx, 2048 / 16	; ss = bx - 2KB
	mov	ss, bx		;

	mov	sp, 2048	; 2KB stack
	mov	bp, sp		;

	; copy ourself to destination

	push	codeSegment	;
	pop	ds		;

	mov	cx, 256		; 256 words
	xor	si, si		;
	xor	di, di		;
	rep	movsw		;

	push	cs		;
	pop	ds		;

	mov	di, 3 + 8
	mov	si, OFFSET ourReadBuf + 3 + 8
	mov	cx, SIZE BPB_FAT32 - 8
	rep	movsb

	; jump to relocated code

	mov	dl, TEST_DRIVE_INT13

	push	es
	push	OFFSET mainEntryPoint

	int	3

	retf

ourInt18Handler:
ourInt19Handler:

	mov	ax, 2518h
	lds	dx, cs:oldInt18
	int	21h

	mov	ax, 2519h
	lds	dx, cs:oldInt19
	int	21h

	mov	ax, 4C00h
	int	21h

oldInt18	LABEL	DWORD
oldInt18Ofs	DW	?
oldInt18Seg	DW	?

oldInt19	LABEL	DWORD
oldInt19Ofs	DW	?
oldInt19Seg	DW	?

ourReadBuf	DB	512 DUP (?)

ourStackDef	LABEL	WORD
		DB	512 DUP (?)
ourStackTop	LABEL	WORD

testSegment ENDS

lastSegment SEGMENT USE16

lastSegment ENDS

	END	testEntryPoint
