{=---------------------------------------------------------------------=}
{                                                                       }
{         DEBSYM -- Kernel Debugger Symbol File Creator v1.0a           }
{                                                                       }
{   Copyright (c) 1999-2000 by Thomas Kjoernes, all rights reserved.    }
{                                                                       }
{=---------------------------------------------------------------------=}

{$S-}
{$R-}
{$I-}
{$G+}

PROGRAM KernelDebuggerSymbolFileCreator;

USES
  Dos;

TYPE
  TSymHeader = RECORD
    Signature: ARRAY [1..4] OF Char;
    Version: Word;
    Flags: Word;
    Owner: ARRAY [1..8] OF Char;
    TableStart: Longint;
    TableCount: Longint;
    SymbolBase: Longint;
    SymbolSize: Longint;
  END;

  TSymTableEntry = RECORD
    SymID: Longint;
    SymNamePtr: Longint;
  END;

VAR
  CopyBuf: ARRAY [1..32768] OF Char;
  SymHeader: TSymHeader;
  SymTableEntry: TSymTableEntry;
  NumRead: Word;
  DevIncPath: PathStr;
  DevBinPath: PathStr;
  StringData: String;
  SearchInfo: SearchRec;
  TmpFile: FILE;
  SymFile: FILE;
  IncFile: Text;
  LineBuf: String;
  TempBuf: String;
  L: Longint;
  I, J: Integer;

FUNCTION Upper(S: String): String;

VAR
  I: Integer;
  T: String;

BEGIN
  T[0] := S[0];
  FOR I := 1 TO Length(S) DO T[I] := UpCase(S[I]);
  Upper := T;
END;

PROCEDURE TrimR(VAR S: String);

BEGIN
  WHILE (S <> '') AND ((S[Length(S)] = ' ') OR (S[Length(S)] = #9)) DO Dec(S[0]);
END;

PROCEDURE TrimL(VAR S: String);

BEGIN
  WHILE (S <> '') AND ((S[1] = '') OR (S[1] = #9)) DO Delete(S, 1, 1);
END;

PROCEDURE Fail(Msg: String);
BEGIN
  WriteLn('Fatal: ', Msg);
  Halt(1);
END;

BEGIN
  WriteLn('Kernel Debugger Symbol File Creator v1.0a');
  WriteLn('Copyright (c) 1999-2000 by Thomas Kjoernes, all rights reserved.');
  DevIncPath := GetEnv('DEVINC') + '\';
  IF DevIncPath = '' THEN Fail('DEVINC= variable is not set.');
  DevBinPath := GetEnv('DEVDIR') + '\bin\SYMBOLS.SYM';
  IF DevBinPath = '' THEN Fail('DEVDIR= variable is not set.');
  FindFirst(DevIncPath + '*.API', 0, SearchInfo);
  IF DosError <> 0 THEN Fail('There are no *.API files in "'+DevIncPath+'".') ELSE
  BEGIN
    WriteLn;
    WriteLn('Processing API files...');
    WriteLn;
    Assign(TmpFile, '~~TMP~~.$$$');
    Rewrite(TmpFile, 1);
    Assign(SymFile, DevBinPath);
    Rewrite(SymFile, 1);
    SymHeader.Signature := 'KDSF';
    SymHeader.Version := $0100;
    SymHeader.Flags := $0000;
    SymHeader.TableStart := SizeOf(SymHeader);
    SymHeader.TableCount := 0;
    BlockWrite(SymFile, SymHeader, SizeOf(SymHeader));
    WHILE DosError = 0 DO
    BEGIN
      WriteLn('  ',DevIncPath+SearchInfo.Name);
      Assign(IncFile, DevIncPath+SearchInfo.Name);
      {$I-}
      Reset(IncFile);
      {$I+}
      IF IOResult = 0 THEN
      BEGIN
        WHILE NOT Eof(IncFile) DO
        BEGIN
          ReadLn(IncFile, LineBuf);
          I := Pos(';', LineBuf);
          IF I <> 0 THEN Delete(LineBuf, I, 255);
          IF LineBuf <> '' THEN
          BEGIN
            TempBuf := Upper(LineBuf);
            I := Pos('EQU ', TempBuf);
            IF I = 0 THEN I := Pos('EQU'#9, TempBuf);
            IF I <> 0 THEN
            BEGIN
              TempBuf := Copy(LineBuf, I+3, 255);
              TrimL(TempBuf);
              TrimR(TempBuf);
              TempBuf := '$' + TempBuf;
              IF UpCase(TempBuf[Length(TempBuf)]) = 'H' THEN Dec(TempBuf[0]);
              Val(TempBuf, L, J);
              IF J <> 0 THEN WriteLn('Warning: Invalid API Number (', TempBuf,') [',J,']');
              SymTableEntry.SymID := L;
              SymTableEntry.SymNamePtr := FileSize(TmpFile);
              BlockWrite(SymFile, SymTableEntry, SizeOf(SymTableEntry));
              TempBuf := Copy(LineBuf, 1, I-1);
              TrimL(TempBuf);
              TrimR(TempBuf);
              TempBuf := TempBuf + #0;
              BlockWrite(TmpFile, TempBuf[1], Length(TempBuf));
              Inc(SymHeader.TableCount);
            END;
          END;
        END;
        Close(IncFile);
      END;
      FindNext(SearchInfo);
    END;
    Seek(TmpFile, 0);
    Seek(SymFile, 0);
    SymHeader.SymbolBase := FileSize(SymFile);
    SymHeader.SymbolSize := FileSize(TmpFile);
    BlockWrite(SymFile, SymHeader, SizeOf(SymHeader));
    Seek(SymFile, FileSize(SymFile));
    REPEAT
      BlockRead(TmpFile, CopyBuf, SizeOf(CopyBuf), NumRead);
      BlockWrite(SymFile, CopyBuf, NumRead);
    UNTIL NumRead = 0;
    Close(TmpFile);
    Close(SymFile);
    Erase(TmpFile);
  END;
END.