{=---------------------------------------------------------------------=}
{                                                                       }
{            MZSHRINK -- MZ Executable File Shrinker v1.0a              }
{                                                                       }
{   Copyright (c) 1999-2000 by Thomas Kjoernes, all rights reserved.    }
{                                                                       }
{=---------------------------------------------------------------------=}

{$S-}
{$R-}
{$I-}
{$G+}

PROGRAM MZ_EXE_Shrinker;

TYPE
  TMZHeader = RECORD
    Signature: Word;
    PagesMod: Word;
    PagesDiv: Word;
    RelocCount: Word;
    HeaderSize: Word;
    MinMem: Word;
    MaxMem: Word;
    InitSS: Word;
    InitSP: Word;
    Checksum: Word;
    EntryIP: Word;
    EntryCS: Word;
    RelocTable: Word;
    OverlayNum: Word;
    Reserved: ARRAY [1..28] OF Char;
    LibDirPtr: Longint;
    NewExePtr: Longint;
  END;

  TLibHdr = RECORD
    Signature: ARRAY [1..4] OF Char;
    LibFlags: Longint;
    LibStart: Longint;
    DirCount: Longint;
  END;

VAR
  SHeader: TMZHeader;
  DHeader: TMZHeader;
  SFile: FILE;
  DFile: FILE;
  SPath: String;
  DPath: String;
  LibHdr: TLibHdr;
  Buf: ARRAY [1..32768] OF Char;
  NumRead: Word;
  L: Longint;

PROCEDURE ShrinkExe;
BEGIN

  SPath := ParamStr(1);
  DPath := 'SHRINK.$$$';

  Assign(SFile, SPath);

  {$I-}
  Reset(SFile, 1);
  {$I+}

  IF IOResult <> 0 THEN
  BEGIN
    WriteLn('- Unable to open EXE "',SPath,'"');
    Halt(1);
  END;

  BlockRead(SFile, SHeader, SizeOf(SHeader));

  IF SHeader.Signature <> $5A4D THEN
  BEGIN
    WriteLn('- File is not a valid "MZ" Executable!');
    Halt(2);
  END;

  IF (SHeader.RelocTable >= $0040) AND (SHeader.NewExePtr <> 0) THEN
  BEGIN
    WriteLn('- Cannot shrink NE, LE, LX or PE files!');
    Halt(2);
  END;

  IF (SHeader.LibDirPtr <> 0) AND (SHeader.LibDirPtr < FileSize(SFile)) THEN
  BEGIN
    Seek(SFile, SHeader.LibDirPtr);
    BlockRead(SFile, LibHdr, SizeOf(LibHdr));
    IF LibHdr.Signature = '$LIB' THEN
    BEGIN
      WriteLn('- Cannot shrink EXELIB library files!');
      Halt(2);
    END;
  END;

  Close(SFile);

  Assign(DFile, DPath);
  {$I-}
  Reset(DFile, 1);
  {$I+}
  IF IOResult = 0 THEN
  BEGIN
    Close(DFile);
    Erase(DFile);
  END;

  Rename(SFile, DPath);

  Assign(SFile, DPath);
  {$I-}
  Reset(SFile, 1);
  {$I+}

  Assign(DFile, SPath);
  {$I-}
  Rewrite(DFile, 1);
  {$I+}

  DHeader := SHeader;

  FillChar(DHeader.Reserved, SizeOf(DHeader.Reserved), 0);

  DHeader.LibDirPtr := 0;
  DHeader.NewExePtr := 0;

  DHeader.HeaderSize := (SizeOf(DHeader) + (DHeader.RelocCount * 4) + 15) DIV 16;

  IF DHeader.RelocCount = 0 THEN DHeader.RelocTable := 0 ELSE DHeader.RelocTable := SizeOf(DHeader);

  BlockWrite(DFile, DHeader, SizeOf(DHeader));

  IF DHeader.RelocCount <> 0 THEN
  BEGIN
    Seek(SFile, SHeader.RelocTable);
    L := DHeader.RelocCount * 4;
    BlockRead(SFile, Buf, L);
    BlockWrite(DFile, Buf, L);
  END;

  L := ((FilePos(DFile) + 15) AND $FFFFFFF0) - FilePos(DFile);

  IF L > 0 THEN
  BEGIN
    FillChar(Buf, L, 0);
    BlockWrite(DFile, Buf, L);
  END;

  Seek(SFile, SHeader.HeaderSize * 16);

  REPEAT
    BlockRead(SFile, Buf, SizeOf(Buf), NumRead);
    BlockWrite(DFile, Buf, NumRead);
  UNTIL NumRead = 0;

  DHeader.PagesDiv := FileSize(DFile) DIV 512;
  DHeader.PagesMod := FileSize(DFile) MOD 512;

  IF DHeader.PagesMod <> 0 THEN Inc(DHeader.PagesDiv);

  Seek(DFile, 0);

  BlockWrite(DFile, DHeader, SizeOf(DHeader));

  Close(SFile);
  Close(DFile);
  Erase(SFile);
END;

BEGIN
  IF ParamCount = 0 THEN
  BEGIN
    WriteLn('MZSHRINK - MZ Executable File Shrinker v1.0a');
    WriteLn('Copyright (c) 1999-2000 by Thomas Kjoernes, all rights reserved.');
    WriteLn;
    WriteLn('  Usage: MZSHRINK [filename.EXE]');
  END ELSE ShrinkExe;
END.